"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.router = exports.Route = exports.Response = exports.Request = exports.Method = void 0;
const http_1 = require("http");
exports.Method = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    DELETE: 'DELETE',
};
class Request extends http_1.IncomingMessage {
}
exports.Request = Request;
class Response extends http_1.ServerResponse {
}
exports.Response = Response;
class Router {
    routes = {};
    constructor() {
        this.routes[exports.Method.GET] = [];
    }
    registerRoutes(callback) {
        callback(this);
    }
    dispatch(req, res) {
        if (req.method === undefined) {
            res.statusCode = 404;
            res.end('404 Not Found');
            return;
        }
        const potentialRoutes = this.routes[req.method].filter(route => route.isMatch(req));
        if (potentialRoutes.length > 0) {
            potentialRoutes[0].action(req, res);
            return;
        }
        res.statusCode = 404;
        res.end('404 Not Found');
    }
    get(path, action) {
        this.routes[exports.Method.GET].push(new Route(exports.Method.GET, path, action));
    }
}
class Route {
    method;
    path;
    action;
    regex;
    constructor(method, path, action) {
        this.method = method;
        this.path = path;
        this.action = action;
        this.regex = this.buildRegex();
    }
    isMatch(req) {
        return req.url !== undefined ? this.regex.test(req.url) : false;
    }
    buildRegex() {
        const segments = this.path.split('/')
            .map(segment => {
            if (segment.charAt(0) === '{' && segment.charAt(segment.length - 1) === '}') {
                return `(?<${segment.slice(1, -1)}>[A-Za-z0-9]+)`;
            }
            return segment;
        })
            .join('/');
        return new RegExp(`^${segments}$`);
    }
}
exports.Route = Route;
exports.router = new Router();
